@@ -111,7 +111,8 @@ __copyright__ = """
 
 __version__ = '1.0.7'
 
-import sys, os, re
+import collections
+import sys, os, re, subprocess
 
 ### Globals & Constants
 
@@ -121,24 +122,28 @@ try:
 except AttributeError:
     # os.devnull was added in Python 2.4, so emulate it for earlier
     # Python versions
-    if sys.platform in ('dos','win32','win16','os2'):
+    if sys.platform in ('dos', 'win32', 'win16'):
         # Use the old CP/M NUL as device name
         DEV_NULL = 'NUL'
     else:
         # Standard Unix uses /dev/null
         DEV_NULL = '/dev/null'
 
+# Directory to search for configuration information on Unix.
+# Constant used by test_platform to test linux_distribution().
+_UNIXCONFDIR = '/etc'
+
 ### Platform specific APIs
 
-_libc_search = re.compile(r'(__libc_init)'
-                          '|'
-                          '(GLIBC_([0-9.]+))'
-                          '|'
-                          '(libc(_\w+)?\.so(?:\.(\d[0-9.]*))?)', re.ASCII)
+_libc_search = re.compile(b'(__libc_init)'
+                          b'|'
+                          b'(GLIBC_([0-9.]+))'
+                          b'|'
+                          br'(libc(_\w+)?\.so(?:\.(\d[0-9.]*))?)', re.ASCII)
 
-def libc_ver(executable=sys.executable,lib='',version='',
+def libc_ver(executable=sys.executable, lib='', version='',
 
-             chunksize=2048):
+             chunksize=16384):
 
     """ Tries to determine the libc version that the file executable
         (which defaults to the Python interpreter) is linked against.
@@ -158,18 +163,23 @@ def libc_ver(executable=sys.executable,lib='',version='',
         # here to work around problems with Cygwin not being
         # able to open symlinks for reading
         executable = os.path.realpath(executable)
-    f = open(executable,'rb')
-    binary = f.read(chunksize).decode('latin-1')
+    f = open(executable, 'rb')
+    binary = f.read(chunksize)
     pos = 0
     while 1:
-        m = _libc_search.search(binary,pos)
+        if b'libc' in binary or b'GLIBC' in binary:
+            m = _libc_search.search(binary, pos)
+        else:
+            m = None
         if not m:
-            binary = f.read(chunksize).decode('latin-1')
+            binary = f.read(chunksize)
             if not binary:
                 break
             pos = 0
             continue
-        libcinit,glibc,glibcversion,so,threads,soversion = m.groups()
+        libcinit, glibc, glibcversion, so, threads, soversion = [
+            s.decode('latin1') if s is not None else s
+            for s in m.groups()]
         if libcinit and not lib:
             lib = 'libc'
         elif glibc:
@@ -187,9 +197,9 @@ def libc_ver(executable=sys.executable,lib='',version='',
                     version = version + threads
         pos = m.end()
     f.close()
-    return lib,version
+    return lib, version
 
-def _dist_try_harder(distname,version,id):
+def _dist_try_harder(distname, version, id):
 
     """ Tries some special tricks to get the distribution
         information in case the default method fails.
@@ -204,7 +214,7 @@ def _dist_try_harder(distname,version,id):
         for line in open('/var/adm/inst-log/info'):
             tv = line.split()
             if len(tv) == 2:
-                tag,value = tv
+                tag, value = tv
             else:
                 continue
             if tag == 'MIN_DIST_VERSION':
@@ -212,7 +222,7 @@ def _dist_try_harder(distname,version,id):
             elif tag == 'DIST_IDENT':
                 values = value.split('-')
                 id = values[2]
-        return distname,version,id
+        return distname, version, id
 
     if os.path.exists('/etc/.installed'):
         # Caldera OpenLinux has some infos in that file (thanks to Colin Kong)
@@ -221,10 +231,10 @@ def _dist_try_harder(distname,version,id):
             if len(pkg) >= 2 and pkg[0] == 'OpenLinux':
                 # XXX does Caldera support non Intel platforms ? If yes,
                 #     where can we find the needed id ?
-                return 'OpenLinux',pkg[1],id
+                return 'OpenLinux', pkg[1], id
 
     if os.path.isdir('/usr/lib/setup'):
-        # Check for slackware verson tag file (thanks to Greg Andruk)
+        # Check for slackware version tag file (thanks to Greg Andruk)
         verfiles = os.listdir('/usr/lib/setup')
         for n in range(len(verfiles)-1, -1, -1):
             if verfiles[n][:14] != 'slack-version-':
@@ -233,9 +243,9 @@ def _dist_try_harder(distname,version,id):
             verfiles.sort()
             distname = 'slackware'
             version = verfiles[-1][14:]
-            return distname,version,id
+            return distname, version, id
 
-    return distname,version,id
+    return distname, version, id
 
 _release_filename = re.compile(r'(\w+)[-_](release|version)', re.ASCII)
 _lsb_release_version = re.compile(r'(.+)'
@@ -255,7 +265,7 @@ _release_version = re.compile(r'([^0-9]+)'
 _supported_dists = (
     'SuSE', 'debian', 'fedora', 'redhat', 'centos',
     'mandrake', 'mandriva', 'rocks', 'slackware', 'yellowdog', 'gentoo',
-    'UnitedLinux', 'turbolinux')
+    'UnitedLinux', 'turbolinux', 'arch', 'mageia')
 
 def _parse_release_file(firstline):
 
@@ -276,7 +286,7 @@ def _parse_release_file(firstline):
     if m is not None:
         return tuple(m.groups())
 
-    # Unkown format... take the first two words
+    # Unknown format... take the first two words
     l = firstline.strip().split()
     if l:
         version = l[0]
@@ -304,28 +314,29 @@ def linux_distribution(distname='', version='', id='',
         distribution read from the OS is returned. Otherwise the short
         name taken from supported_dists is used.
 
-        Returns a tuple (distname,version,id) which default to the
+        Returns a tuple (distname, version, id) which default to the
         args given as parameters.
 
     """
     try:
-        etc = os.listdir('/etc')
-    except os.error:
+        etc = os.listdir(_UNIXCONFDIR)
+    except OSError:
         # Probably not a Unix system
-        return distname,version,id
+        return distname, version, id
     etc.sort()
     for file in etc:
         m = _release_filename.match(file)
         if m is not None:
-            _distname,dummy = m.groups()
+            _distname, dummy = m.groups()
             if _distname in supported_dists:
                 distname = _distname
                 break
     else:
-        return _dist_try_harder(distname,version,id)
+        return _dist_try_harder(distname, version, id)
 
     # Read the first line
-    with open('/etc/'+file, 'r') as f:
+    with open(os.path.join(_UNIXCONFDIR, file), 'r',
+              encoding='utf-8', errors='surrogateescape') as f:
         firstline = f.readline()
     _distname, _version, _id = _parse_release_file(firstline)
 
@@ -339,7 +350,7 @@ def linux_distribution(distname='', version='', id='',
 
 # To maintain backwards compatibility:
 
-def dist(distname='',version='',id='',
+def dist(distname='', version='', id='',
 
          supported_dists=_supported_dists):
 
@@ -349,7 +360,7 @@ def dist(distname='',version='',id='',
         /etc and then reverts to _dist_try_harder() in case no
         suitable files are found.
 
-        Returns a tuple (distname,version,id) which default to the
+        Returns a tuple (distname, version, id) which default to the
         args given as parameters.
 
     """
@@ -357,92 +368,13 @@ def dist(distname='',version='',id='',
                               supported_dists=supported_dists,
                               full_distribution_name=0)
 
-class _popen:
-
-    """ Fairly portable (alternative) popen implementation.
-
-        This is mostly needed in case os.popen() is not available, or
-        doesn't work as advertised, e.g. in Win9X GUI programs like
-        PythonWin or IDLE.
-
-        Writing to the pipe is currently not supported.
-
-    """
-    tmpfile = ''
-    pipe = None
-    bufsize = None
-    mode = 'r'
-
-    def __init__(self,cmd,mode='r',bufsize=None):
-
-        if mode != 'r':
-            raise ValueError('popen()-emulation only supports read mode')
-        import tempfile
-        self.tmpfile = tmpfile = tempfile.mktemp()
-        os.system(cmd + ' > %s' % tmpfile)
-        self.pipe = open(tmpfile,'rb')
-        self.bufsize = bufsize
-        self.mode = mode
-
-    def read(self):
-
-        return self.pipe.read()
-
-    def readlines(self):
-
-        if self.bufsize is not None:
-            return self.pipe.readlines()
-
-    def close(self,
-
-              remove=os.unlink,error=os.error):
-
-        if self.pipe:
-            rc = self.pipe.close()
-        else:
-            rc = 255
-        if self.tmpfile:
-            try:
-                remove(self.tmpfile)
-            except error:
-                pass
-        return rc
-
-    # Alias
-    __del__ = close
-
 def popen(cmd, mode='r', bufsize=-1):
 
     """ Portable popen() interface.
     """
-    # Find a working popen implementation preferring win32pipe.popen
-    # over os.popen over _popen
-    popen = None
-    if os.environ.get('OS','') == 'Windows_NT':
-        # On NT win32pipe should work; on Win9x it hangs due to bugs
-        # in the MS C lib (see MS KnowledgeBase article Q150956)
-        try:
-            import win32pipe
-        except ImportError:
-            pass
-        else:
-            popen = win32pipe.popen
-    if popen is None:
-        if hasattr(os,'popen'):
-            popen = os.popen
-            # Check whether it works... it doesn't in GUI programs
-            # on Windows platforms
-            if sys.platform == 'win32': # XXX Others too ?
-                try:
-                    popen('')
-                except os.error:
-                    popen = _popen
-        else:
-            popen = _popen
-    if bufsize is None:
-        return popen(cmd,mode)
-    else:
-        return popen(cmd,mode,bufsize)
+    import warnings
+    warnings.warn('use os.popen instead', DeprecationWarning, stacklevel=2)
+    return os.popen(cmd, mode, bufsize)
 
 def _norm_version(version, build=''):
 
@@ -453,11 +385,11 @@ def _norm_version(version, build=''):
     if build:
         l.append(build)
     try:
-        ints = map(int,l)
+        ints = map(int, l)
     except ValueError:
         strings = l
     else:
-        strings = list(map(str,ints))
+        strings = list(map(str, ints))
     version = '.'.join(strings[:3])
     return version
 
@@ -476,46 +408,43 @@ _ver_output = re.compile(r'(?:([\w ]+) ([\w.]+) '
 
 def _syscmd_ver(system='', release='', version='',
 
-               supported_platforms=('win32','win16','dos','os2')):
+               supported_platforms=('win32', 'win16', 'dos')):
 
     """ Tries to figure out the OS version used and returns
-        a tuple (system,release,version).
+        a tuple (system, release, version).
 
         It uses the "ver" shell command for this which is known
-        to exists on Windows, DOS and OS/2. XXX Others too ?
+        to exists on Windows, DOS. XXX Others too ?
 
         In case this fails, the given parameters are used as
         defaults.
 
     """
     if sys.platform not in supported_platforms:
-        return system,release,version
+        return system, release, version
 
     # Try some common cmd strings
-    for cmd in ('ver','command /c ver','cmd /c ver'):
+    for cmd in ('ver', 'command /c ver', 'cmd /c ver'):
         try:
             pipe = popen(cmd)
             info = pipe.read()
             if pipe.close():
-                raise os.error('command failed')
+                raise OSError('command failed')
             # XXX How can I suppress shell errors from being written
             #     to stderr ?
-        except os.error as why:
-            #print 'Command %s failed: %s' % (cmd,why)
-            continue
-        except IOError as why:
-            #print 'Command %s failed: %s' % (cmd,why)
+        except OSError as why:
+            #print 'Command %s failed: %s' % (cmd, why)
             continue
         else:
             break
     else:
-        return system,release,version
+        return system, release, version
 
     # Parse the output
     info = info.strip()
     m = _ver_output.match(info)
     if m is not None:
-        system,release,version = m.groups()
+        system, release, version = m.groups()
         # Strip trailing dots from version and release
         if release[-1] == '.':
             release = release[:-1]
@@ -524,9 +453,9 @@ def _syscmd_ver(system='', release='', version='',
         # Normalize the version and build strings (eliminating additional
         # zeros)
         version = _norm_version(version)
-    return system,release,version
+    return system, release, version
 
-def _win32_getvalue(key,name,default=''):
+def _win32_getvalue(key, name, default=''):
 
     """ Read a value for name from the registry key.
 
@@ -541,14 +470,14 @@ def _win32_getvalue(key,name,default=''):
         import winreg
         RegQueryValueEx = winreg.QueryValueEx
     try:
-        return RegQueryValueEx(key,name)
+        return RegQueryValueEx(key, name)
     except:
         return default
 
-def win32_ver(release='',version='',csd='',ptype=''):
+def win32_ver(release='', version='', csd='', ptype=''):
 
     """ Get additional version information from the Windows Registry
-        and return a tuple (version,csd,ptype) referring to version
+        and return a tuple (version, csd, ptype) referring to version
         number, CSD level (service pack), and OS type (multi/single
         processor).
 
@@ -585,7 +514,7 @@ def win32_ver(release='',version='',csd='',ptype=''):
             sys.getwindowsversion
         except AttributeError:
             # No emulation possible, so return the defaults...
-            return release,version,csd,ptype
+            return release, version, csd, ptype
         else:
             # Emulation using winreg (added in Python 2.0) and
             # sys.getwindowsversion() (added in Python 2.3)
@@ -603,8 +532,8 @@ def win32_ver(release='',version='',csd='',ptype=''):
 
     # Find out the registry key and some general version infos
     winver = GetVersionEx()
-    maj,min,buildno,plat,csd = winver
-    version = '%i.%i.%i' % (maj,min,buildno & 0xFFFF)
+    maj, min, buildno, plat, csd = winver
+    version = '%i.%i.%i' % (maj, min, buildno & 0xFFFF)
     if hasattr(winver, "service_pack"):
         if winver.service_pack != "":
             csd = 'SP%s' % winver.service_pack_major
@@ -654,7 +583,7 @@ def win32_ver(release='',version='',csd='',ptype=''):
                     # Discard any type that isn't REG_SZ
                     if type == REG_SZ and name.find("Server") != -1:
                         product_type = VER_NT_SERVER
-                except WindowsError:
+                except OSError:
                     # Use default of VER_NT_WORKSTATION
                     pass
 
@@ -668,14 +597,19 @@ def win32_ver(release='',version='',csd='',ptype=''):
                     release = '7'
                 else:
                     release = '2008ServerR2'
+            elif min == 2:
+                if product_type == VER_NT_WORKSTATION:
+                    release = '8'
+                else:
+                    release = '2012Server'
             else:
-                release = 'post2008Server'
+                release = 'post2012Server'
 
     else:
         if not release:
             # E.g. Win3.1 with win32s
-            release = '%i.%i' % (maj,min)
-        return release,version,csd,ptype
+            release = '%i.%i' % (maj, min)
+        return release, version, csd, ptype
 
     # Open the registry key
     try:
@@ -683,7 +617,7 @@ def win32_ver(release='',version='',csd='',ptype=''):
         # Get a value to make sure the key exists...
         RegQueryValueEx(keyCurVer, 'SystemRoot')
     except:
-        return release,version,csd,ptype
+        return release, version, csd, ptype
 
     # Parse values
     #subversion = _win32_getvalue(keyCurVer,
@@ -693,73 +627,17 @@ def win32_ver(release='',version='',csd='',ptype=''):
     #   release = release + subversion # 95a, 95b, etc.
     build = _win32_getvalue(keyCurVer,
                             'CurrentBuildNumber',
-                            ('',1))[0]
+                            ('', 1))[0]
     ptype = _win32_getvalue(keyCurVer,
                            'CurrentType',
-                           (ptype,1))[0]
+                           (ptype, 1))[0]
 
     # Normalize version
-    version = _norm_version(version,build)
+    version = _norm_version(version, build)
 
     # Close key
     RegCloseKey(keyCurVer)
-    return release,version,csd,ptype
-
-def _mac_ver_lookup(selectors,default=None):
-
-    from _gestalt import gestalt
-    l = []
-    append = l.append
-    for selector in selectors:
-        try:
-            append(gestalt(selector))
-        except (RuntimeError, OSError):
-            append(default)
-    return l
-
-def _bcd2str(bcd):
-
-    return hex(bcd)[2:]
-
-def _mac_ver_gestalt():
-    """
-        Thanks to Mark R. Levinson for mailing documentation links and
-        code examples for this function. Documentation for the
-        gestalt() API is available online at:
-
-           http://www.rgaros.nl/gestalt/
-    """
-    # Check whether the version info module is available
-    try:
-        import _gestalt
-    except ImportError:
-        return None
-    # Get the infos
-    sysv, sysa = _mac_ver_lookup(('sysv','sysa'))
-    # Decode the infos
-    if sysv:
-        major = (sysv & 0xFF00) >> 8
-        minor = (sysv & 0x00F0) >> 4
-        patch = (sysv & 0x000F)
-
-        if (major, minor) >= (10, 4):
-            # the 'sysv' gestald cannot return patchlevels
-            # higher than 9. Apple introduced 3 new
-            # gestalt codes in 10.4 to deal with this
-            # issue (needed because patch levels can
-            # run higher than 9, such as 10.4.11)
-            major,minor,patch = _mac_ver_lookup(('sys1','sys2','sys3'))
-            release = '%i.%i.%i' %(major, minor, patch)
-        else:
-            release = '%s.%i.%i' % (_bcd2str(major),minor,patch)
-
-    if sysa:
-        machine = {0x1: '68k',
-                   0x2: 'PowerPC',
-                   0xa: 'i386'}.get(sysa,'')
-
-    versioninfo=('', '', '')
-    return release,versioninfo,machine
+    return release, version, csd, ptype
 
 def _mac_ver_xml():
     fn = '/System/Library/CoreServices/SystemVersion.plist'
@@ -771,24 +649,25 @@ def _mac_ver_xml():
     except ImportError:
         return None
 
-    pl = plistlib.readPlist(fn)
+    with open(fn, 'rb') as f:
+        pl = plistlib.load(f)
     release = pl['ProductVersion']
-    versioninfo=('', '', '')
-    machine = os.uname()[4]
+    versioninfo = ('', '', '')
+    machine = os.uname().machine
     if machine in ('ppc', 'Power Macintosh'):
-        # for compatibility with the gestalt based code
+        # Canonical name
         machine = 'PowerPC'
 
-    return release,versioninfo,machine
+    return release, versioninfo, machine
 
 
-def mac_ver(release='',versioninfo=('','',''),machine=''):
+def mac_ver(release='', versioninfo=('', '', ''), machine=''):
 
     """ Get MacOS version information and return it as tuple (release,
         versioninfo, machine) with versioninfo being a tuple (version,
         dev_stage, non_release_version).
 
-        Entries which cannot be determined are set to the paramter values
+        Entries which cannot be determined are set to the parameter values
         which default to ''. All tuple entries are strings.
     """
 
@@ -798,16 +677,10 @@ def mac_ver(release='',versioninfo=('','',''),machine=''):
     if info is not None:
         return info
 
-    # If that doesn't work for some reason fall back to reading the
-    # information using gestalt calls.
-    info = _mac_ver_gestalt()
-    if info is not None:
-        return info
-
     # If that also doesn't work return the default values
-    return release,versioninfo,machine
+    return release, versioninfo, machine
 
-def _java_getprop(name,default):
+def _java_getprop(name, default):
 
     from java.lang import System
     try:
@@ -818,13 +691,13 @@ def _java_getprop(name,default):
     except AttributeError:
         return default
 
-def java_ver(release='',vendor='',vminfo=('','',''),osinfo=('','','')):
+def java_ver(release='', vendor='', vminfo=('', '', ''), osinfo=('', '', '')):
 
     """ Version interface for Jython.
 
-        Returns a tuple (release,vendor,vminfo,osinfo) with vminfo being
-        a tuple (vm_name,vm_release,vm_vendor) and osinfo being a
-        tuple (os_name,os_version,os_arch).
+        Returns a tuple (release, vendor, vminfo, osinfo) with vminfo being
+        a tuple (vm_name, vm_release, vm_vendor) and osinfo being a
+        tuple (os_name, os_version, os_arch).
 
         Values which cannot be determined are set to the defaults
         given as parameters (which all default to '').
@@ -834,7 +707,7 @@ def java_ver(release='',vendor='',vminfo=('','',''),osinfo=('','','')):
     try:
         import java.lang
     except ImportError:
-        return release,vendor,vminfo,osinfo
+        return release, vendor, vminfo, osinfo
 
     vendor = _java_getprop('java.vendor', vendor)
     release = _java_getprop('java.version', release)
@@ -853,9 +726,9 @@ def java_ver(release='',vendor='',vminfo=('','',''),osinfo=('','','')):
 
 ### System name aliasing
 
-def system_alias(system,release,version):
+def system_alias(system, release, version):
 
-    """ Returns (system,release,version) aliased to common
+    """ Returns (system, release, version) aliased to common
         marketing names used for some systems.
 
         It also does some reordering of the information in some cases
@@ -865,13 +738,13 @@ def system_alias(system,release,version):
     if system == 'Rhapsody':
         # Apple's BSD derivative
         # XXX How can we determine the marketing release number ?
-        return 'MacOS X Server',system+release,version
+        return 'MacOS X Server', system+release, version
 
     elif system == 'SunOS':
         # Sun's OS
         if release < '5':
             # These releases use the old name SunOS
-            return system,release,version
+            return system, release, version
         # Modify release (marketing release = SunOS release - 3)
         l = release.split('.')
         if l:
@@ -899,11 +772,11 @@ def system_alias(system,release,version):
         else:
             version = '64bit'
 
-    elif system in ('win32','win16'):
+    elif system in ('win32', 'win16'):
         # In case one of the other tricks
         system = 'Windows'
 
-    return system,release,version
+    return system, release, version
 
 ### Various internal helpers
 
@@ -916,21 +789,21 @@ def _platform(*args):
     platform = '-'.join(x.strip() for x in filter(len, args))
 
     # Cleanup some possible filename obstacles...
-    platform = platform.replace(' ','_')
-    platform = platform.replace('/','-')
-    platform = platform.replace('\\','-')
-    platform = platform.replace(':','-')
-    platform = platform.replace(';','-')
-    platform = platform.replace('"','-')
-    platform = platform.replace('(','-')
-    platform = platform.replace(')','-')
+    platform = platform.replace(' ', '_')
+    platform = platform.replace('/', '-')
+    platform = platform.replace('\\', '-')
+    platform = platform.replace(':', '-')
+    platform = platform.replace(';', '-')
+    platform = platform.replace('"', '-')
+    platform = platform.replace('(', '-')
+    platform = platform.replace(')', '-')
 
     # No need to report 'unknown' information...
-    platform = platform.replace('unknown','')
+    platform = platform.replace('unknown', '')
 
     # Fold '--'s and remove trailing '-'
     while 1:
-        cleaned = platform.replace('--','-')
+        cleaned = platform.replace('--', '-')
         if cleaned == platform:
             break
         platform = cleaned
@@ -950,7 +823,7 @@ def _node(default=''):
         return default
     try:
         return socket.gethostname()
-    except socket.error:
+    except OSError:
         # Still not working...
         return default
 
@@ -962,19 +835,19 @@ def _follow_symlinks(filepath):
     filepath = os.path.abspath(filepath)
     while os.path.islink(filepath):
         filepath = os.path.normpath(
-            os.path.join(os.path.dirname(filepath),os.readlink(filepath)))
+            os.path.join(os.path.dirname(filepath), os.readlink(filepath)))
     return filepath
 
-def _syscmd_uname(option,default=''):
+def _syscmd_uname(option, default=''):
 
     """ Interface to the system's uname command.
     """
-    if sys.platform in ('dos','win32','win16','os2'):
+    if sys.platform in ('dos', 'win32', 'win16'):
         # XXX Others too ?
         return default
     try:
         f = os.popen('uname %s 2> %s' % (option, DEV_NULL))
-    except (AttributeError,os.error):
+    except (AttributeError, OSError):
         return default
     output = f.read().strip()
     rc = f.close()
@@ -983,7 +856,7 @@ def _syscmd_uname(option,default=''):
     else:
         return output
 
-def _syscmd_file(target,default=''):
+def _syscmd_file(target, default=''):
 
     """ Interface to the system's file command.
 
@@ -992,16 +865,18 @@ def _syscmd_file(target,default=''):
         default in case the command should fail.
 
     """
-    if sys.platform in ('dos','win32','win16','os2'):
+    if sys.platform in ('dos', 'win32', 'win16'):
         # XXX Others too ?
         return default
-    target = _follow_symlinks(target).replace('"', '\\"')
+    target = _follow_symlinks(target)
     try:
-        f = os.popen('file -b "%s" 2> %s' % (target, DEV_NULL))
-    except (AttributeError,os.error):
+        proc = subprocess.Popen(['file', target],
+                stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+
+    except (AttributeError, OSError):
         return default
-    output = f.read().strip()
-    rc = f.close()
+    output = proc.communicate()[0].decode('latin-1')
+    rc = proc.wait()
     if not output or rc:
         return default
     else:
@@ -1012,17 +887,17 @@ def _syscmd_file(target,default=''):
 # Default values for architecture; non-empty strings override the
 # defaults given as parameters
 _default_architecture = {
-    'win32': ('','WindowsPE'),
-    'win16': ('','Windows'),
-    'dos': ('','MSDOS'),
+    'win32': ('', 'WindowsPE'),
+    'win16': ('', 'Windows'),
+    'dos': ('', 'MSDOS'),
 }
 
-def architecture(executable=sys.executable,bits='',linkage=''):
+def architecture(executable=sys.executable, bits='', linkage=''):
 
     """ Queries the given executable (defaults to the Python interpreter
         binary) for various architecture information.
 
-        Returns a tuple (bits,linkage) which contains information about
+        Returns a tuple (bits, linkage) which contains information about
         the bit architecture and the linkage format used for the
         executable. Both values are returned as strings.
 
@@ -1060,16 +935,16 @@ def architecture(executable=sys.executable,bits='',linkage=''):
         # "file" command did not return anything; we'll try to provide
         # some sensible defaults then...
         if sys.platform in _default_architecture:
-            b,l = _default_architecture[sys.platform]
+            b, l = _default_architecture[sys.platform]
             if b:
                 bits = b
             if l:
                 linkage = l
-        return bits,linkage
+        return bits, linkage
 
     if 'executable' not in fileout:
         # Format not supported
-        return bits,linkage
+        return bits, linkage
 
     # Bits
     if '32-bit' in fileout:
@@ -1097,16 +972,19 @@ def architecture(executable=sys.executable,bits='',linkage=''):
         # XXX the A.OUT format also falls under this class...
         pass
 
-    return bits,linkage
+    return bits, linkage
 
 ### Portable uname() interface
 
+uname_result = collections.namedtuple("uname_result",
+                    "system node release version machine processor")
+
 _uname_cache = None
 
 def uname():
 
     """ Fairly portable uname interface. Returns a tuple
-        of strings (system,node,release,version,machine,processor)
+        of strings (system, node, release, version, machine, processor)
         identifying the underlying platform.
 
         Note that unlike the os.uname function this also returns
@@ -1125,7 +1003,7 @@ def uname():
 
     # Get some infos from the builtin os.uname API...
     try:
-        system,node,release,version,machine = os.uname()
+        system, node, release, version, machine = os.uname()
     except AttributeError:
         no_os_uname = 1
 
@@ -1143,7 +1021,7 @@ def uname():
 
         # Try win32_ver() on win32 platforms
         if system == 'win32':
-            release,version,csd,ptype = win32_ver()
+            release, version, csd, ptype = win32_ver()
             if release and version:
                 use_syscmd_ver = 0
             # Try to use the PROCESSOR_* environment variables
@@ -1162,7 +1040,7 @@ def uname():
         # Try the 'ver' system command available on some
         # platforms
         if use_syscmd_ver:
-            system,release,version = _syscmd_ver(system)
+            system, release, version = _syscmd_ver(system)
             # Normalize system to what win32_ver() normally returns
             # (_syscmd_ver() tends to return the vendor name as well)
             if system == 'Microsoft Windows':
@@ -1180,7 +1058,7 @@ def uname():
 
         # In case we still don't know anything useful, we'll try to
         # help ourselves
-        if system in ('win32','win16'):
+        if system in ('win32', 'win16'):
             if not version:
                 if system == 'win32':
                     version = '32bit'
@@ -1189,7 +1067,7 @@ def uname():
             system = 'Windows'
 
         elif system[:4] == 'java':
-            release,vendor,vminfo,osinfo = java_ver()
+            release, vendor, vminfo, osinfo = java_ver()
             system = 'Java'
             version = ', '.join(vminfo)
             if not version:
@@ -1207,14 +1085,14 @@ def uname():
         except ImportError:
             pass
         else:
-            csid, cpu_number = vms_lib.getsyi('SYI$_CPU',0)
+            csid, cpu_number = vms_lib.getsyi('SYI$_CPU', 0)
             if (cpu_number >= 128):
                 processor = 'Alpha'
             else:
                 processor = 'VAX'
     if not processor:
         # Get processor information from the uname system command
-        processor = _syscmd_uname('-p','')
+        processor = _syscmd_uname('-p', '')
 
     #If any unknowns still exist, replace them with ''s, which are more portable
     if system == 'unknown':
@@ -1235,7 +1113,8 @@ def uname():
         system = 'Windows'
         release = 'Vista'
 
-    _uname_cache = system,node,release,version,machine,processor
+    _uname_cache = uname_result(system, node, release, version,
+                                machine, processor)
     return _uname_cache
 
 ### Direct interfaces to some of the uname() return values
@@ -1247,7 +1126,7 @@ def system():
         An empty string is returned if the value cannot be determined.
 
     """
-    return uname()[0]
+    return uname().system
 
 def node():
 
@@ -1257,7 +1136,7 @@ def node():
         An empty string is returned if the value cannot be determined.
 
     """
-    return uname()[1]
+    return uname().node
 
 def release():
 
@@ -1266,7 +1145,7 @@ def release():
         An empty string is returned if the value cannot be determined.
 
     """
-    return uname()[2]
+    return uname().release
 
 def version():
 
@@ -1275,7 +1154,7 @@ def version():
         An empty string is returned if the value cannot be determined.
 
     """
-    return uname()[3]
+    return uname().version
 
 def machine():
 
@@ -1284,7 +1163,7 @@ def machine():
         An empty string is returned if the value cannot be determined.
 
     """
-    return uname()[4]
+    return uname().machine
 
 def processor():
 
@@ -1296,7 +1175,7 @@ def processor():
         e.g.  NetBSD does this.
 
     """
-    return uname()[5]
+    return uname().processor
 
 ### Various APIs for extracting information from sys.version
 
@@ -1311,6 +1190,14 @@ _ironpython_sys_version_parser = re.compile(
     '(?: \(([\d\.]+)\))?'
     ' on (.NET [\d\.]+)', re.ASCII)
 
+# IronPython covering 2.6 and 2.7
+_ironpython26_sys_version_parser = re.compile(
+    r'([\d.]+)\s*'
+    '\(IronPython\s*'
+    '[\d.]+\s*'
+    '\(([\d.]+)\) on ([\w.]+ [\d.]+(?: \(\d+-bit\))?)\)'
+)
+
 _pypy_sys_version_parser = re.compile(
     r'([\w.+]+)\s*'
     '\(#?([^,]+),\s*([\w ]+),\s*([\w :]+)\)\s*'
@@ -1348,19 +1235,24 @@ def _sys_version(sys_version=None):
         return result
 
     # Parse it
-    if sys_version[:10] == 'IronPython':
+    if 'IronPython' in sys_version:
         # IronPython
         name = 'IronPython'
-        match = _ironpython_sys_version_parser.match(sys_version)
+        if sys_version.startswith('IronPython'):
+            match = _ironpython_sys_version_parser.match(sys_version)
+        else:
+            match = _ironpython26_sys_version_parser.match(sys_version)
+
         if match is None:
             raise ValueError(
                 'failed to parse IronPython sys.version: %s' %
                 repr(sys_version))
+
         version, alt_version, compiler = match.groups()
         buildno = ''
         builddate = ''
 
-    elif sys.platform[:4] == 'java':
+    elif sys.platform.startswith('java'):
         # Jython
         name = 'Jython'
         match = _sys_version_parser.match(sys_version)
@@ -1519,57 +1411,58 @@ def platform(aliased=0, terse=0):
 
     # Get uname information and then apply platform specific cosmetics
     # to it...
-    system,node,release,version,machine,processor = uname()
+    system, node, release, version, machine, processor = uname()
     if machine == processor:
         processor = ''
     if aliased:
-        system,release,version = system_alias(system,release,version)
+        system, release, version = system_alias(system, release, version)
 
     if system == 'Windows':
         # MS platforms
-        rel,vers,csd,ptype = win32_ver(version)
+        rel, vers, csd, ptype = win32_ver(version)
         if terse:
-            platform = _platform(system,release)
+            platform = _platform(system, release)
         else:
-            platform = _platform(system,release,version,csd)
+            platform = _platform(system, release, version, csd)
 
     elif system in ('Linux',):
         # Linux based systems
-        distname,distversion,distid = dist('')
+        distname, distversion, distid = dist('')
         if distname and not terse:
-            platform = _platform(system,release,machine,processor,
+            platform = _platform(system, release, machine, processor,
                                  'with',
-                                 distname,distversion,distid)
+                                 distname, distversion, distid)
         else:
             # If the distribution name is unknown check for libc vs. glibc
-            libcname,libcversion = libc_ver(sys.executable)
-            platform = _platform(system,release,machine,processor,
+            libcname, libcversion = libc_ver(sys.executable)
+            platform = _platform(system, release, machine, processor,
                                  'with',
                                  libcname+libcversion)
     elif system == 'Java':
         # Java platforms
-        r,v,vminfo,(os_name,os_version,os_arch) = java_ver()
+        r, v, vminfo, (os_name, os_version, os_arch) = java_ver()
         if terse or not os_name:
-            platform = _platform(system,release,version)
+            platform = _platform(system, release, version)
         else:
-            platform = _platform(system,release,version,
+            platform = _platform(system, release, version,
                                  'on',
-                                 os_name,os_version,os_arch)
+                                 os_name, os_version, os_arch)
 
     elif system == 'MacOS':
         # MacOS platforms
         if terse:
-            platform = _platform(system,release)
+            platform = _platform(system, release)
         else:
-            platform = _platform(system,release,machine)
+            platform = _platform(system, release, machine)
 
     else:
         # Generic handler
         if terse:
-            platform = _platform(system,release)
+            platform = _platform(system, release)
         else:
-            bits,linkage = architecture(sys.executable)
-            platform = _platform(system,release,machine,processor,bits,linkage)
+            bits, linkage = architecture(sys.executable)
+            platform = _platform(system, release, machine,
+                                 processor, bits, linkage)
 
     _platform_cache[(aliased, terse)] = platform
     return platform
@@ -1580,5 +1473,5 @@ if __name__ == '__main__':
     # Default is to print the aliased verbose platform string
     terse = ('terse' in sys.argv or '--terse' in sys.argv)
     aliased = (not 'nonaliased' in sys.argv and not '--nonaliased' in sys.argv)
-    print(platform(aliased,terse))
+    print(platform(aliased, terse))
     sys.exit(0)
