@@ -3,6 +3,7 @@
 import copy
 import copyreg
 import weakref
+import abc
 from operator import le, lt, ge, gt, eq, ne
 
 import unittest
@@ -17,7 +18,7 @@ class TestCopy(unittest.TestCase):
     # Attempt full line coverage of copy.py from top to bottom
 
     def test_exceptions(self):
-        self.assertTrue(copy.Error is copy.error)
+        self.assertIs(copy.Error, copy.error)
         self.assertTrue(issubclass(copy.Error, Exception))
 
     # The copy() method
@@ -54,20 +55,26 @@ class TestCopy(unittest.TestCase):
     def test_copy_reduce_ex(self):
         class C(object):
             def __reduce_ex__(self, proto):
+                c.append(1)
                 return ""
             def __reduce__(self):
-                raise support.TestFailed("shouldn't call this")
+                self.fail("shouldn't call this")
+        c = []
         x = C()
         y = copy.copy(x)
-        self.assertTrue(y is x)
+        self.assertIs(y, x)
+        self.assertEqual(c, [1])
 
     def test_copy_reduce(self):
         class C(object):
             def __reduce__(self):
+                c.append(1)
                 return ""
+        c = []
         x = C()
         y = copy.copy(x)
-        self.assertTrue(y is x)
+        self.assertIs(y, x)
+        self.assertEqual(c, [1])
 
     def test_copy_cant(self):
         class C(object):
@@ -87,11 +94,13 @@ class TestCopy(unittest.TestCase):
             pass
         def f():
             pass
+        class WithMetaclass(metaclass=abc.ABCMeta):
+            pass
         tests = [None, 42, 2**100, 3.14, True, False, 1j,
                  "hello", "hello\u1234", f.__code__,
-                 NewStyle, range(10), Classic, max]
+                 NewStyle, range(10), Classic, max, WithMetaclass]
         for x in tests:
-            self.assertTrue(copy.copy(x) is x, repr(x))
+            self.assertIs(copy.copy(x), x)
 
     def test_copy_list(self):
         x = [1, 2, 3]
@@ -185,9 +194,9 @@ class TestCopy(unittest.TestCase):
         x = [x, x]
         y = copy.deepcopy(x)
         self.assertEqual(y, x)
-        self.assertTrue(y is not x)
-        self.assertTrue(y[0] is not x[0])
-        self.assertTrue(y[0] is y[1])
+        self.assertIsNot(y, x)
+        self.assertIsNot(y[0], x[0])
+        self.assertIs(y[0], y[1])
 
     def test_deepcopy_issubclass(self):
         # XXX Note: there's no way to test the TypeError coming out of
@@ -227,20 +236,26 @@ class TestCopy(unittest.TestCase):
     def test_deepcopy_reduce_ex(self):
         class C(object):
             def __reduce_ex__(self, proto):
+                c.append(1)
                 return ""
             def __reduce__(self):
-                raise support.TestFailed("shouldn't call this")
+                self.fail("shouldn't call this")
+        c = []
         x = C()
         y = copy.deepcopy(x)
-        self.assertTrue(y is x)
+        self.assertIs(y, x)
+        self.assertEqual(c, [1])
 
     def test_deepcopy_reduce(self):
         class C(object):
             def __reduce__(self):
+                c.append(1)
                 return ""
+        c = []
         x = C()
         y = copy.deepcopy(x)
-        self.assertTrue(y is x)
+        self.assertIs(y, x)
+        self.assertEqual(c, [1])
 
     def test_deepcopy_cant(self):
         class C(object):
@@ -264,14 +279,14 @@ class TestCopy(unittest.TestCase):
                  "hello", "hello\u1234", f.__code__,
                  NewStyle, range(10), Classic, max]
         for x in tests:
-            self.assertTrue(copy.deepcopy(x) is x, repr(x))
+            self.assertIs(copy.deepcopy(x), x)
 
     def test_deepcopy_list(self):
         x = [[1, 2], 3]
         y = copy.deepcopy(x)
         self.assertEqual(y, x)
-        self.assertTrue(x is not y)
-        self.assertTrue(x[0] is not y[0])
+        self.assertIsNot(x, y)
+        self.assertIsNot(x[0], y[0])
 
     def test_deepcopy_reflexive_list(self):
         x = []
@@ -279,16 +294,26 @@ class TestCopy(unittest.TestCase):
         y = copy.deepcopy(x)
         for op in comparisons:
             self.assertRaises(RuntimeError, op, y, x)
-        self.assertTrue(y is not x)
-        self.assertTrue(y[0] is y)
+        self.assertIsNot(y, x)
+        self.assertIs(y[0], y)
         self.assertEqual(len(y), 1)
 
+    def test_deepcopy_empty_tuple(self):
+        x = ()
+        y = copy.deepcopy(x)
+        self.assertIs(x, y)
+
     def test_deepcopy_tuple(self):
         x = ([1, 2], 3)
         y = copy.deepcopy(x)
         self.assertEqual(y, x)
-        self.assertTrue(x is not y)
-        self.assertTrue(x[0] is not y[0])
+        self.assertIsNot(x, y)
+        self.assertIsNot(x[0], y[0])
+
+    def test_deepcopy_tuple_of_immutables(self):
+        x = ((1, 2), 3)
+        y = copy.deepcopy(x)
+        self.assertIs(x, y)
 
     def test_deepcopy_reflexive_tuple(self):
         x = ([],)
@@ -296,16 +321,16 @@ class TestCopy(unittest.TestCase):
         y = copy.deepcopy(x)
         for op in comparisons:
             self.assertRaises(RuntimeError, op, y, x)
-        self.assertTrue(y is not x)
-        self.assertTrue(y[0] is not x[0])
-        self.assertTrue(y[0][0] is y)
+        self.assertIsNot(y, x)
+        self.assertIsNot(y[0], x[0])
+        self.assertIs(y[0][0], y)
 
     def test_deepcopy_dict(self):
         x = {"foo": [1, 2], "bar": 3}
         y = copy.deepcopy(x)
         self.assertEqual(y, x)
-        self.assertTrue(x is not y)
-        self.assertTrue(x["foo"] is not y["foo"])
+        self.assertIsNot(x, y)
+        self.assertIsNot(x["foo"], y["foo"])
 
     def test_deepcopy_reflexive_dict(self):
         x = {}
@@ -315,15 +340,30 @@ class TestCopy(unittest.TestCase):
             self.assertRaises(TypeError, op, y, x)
         for op in equality_comparisons:
             self.assertRaises(RuntimeError, op, y, x)
-        self.assertTrue(y is not x)
-        self.assertTrue(y['foo'] is y)
+        self.assertIsNot(y, x)
+        self.assertIs(y['foo'], y)
         self.assertEqual(len(y), 1)
 
     def test_deepcopy_keepalive(self):
         memo = {}
-        x = 42
+        x = []
         y = copy.deepcopy(x, memo)
-        self.assertTrue(memo[id(x)] is x)
+        self.assertIs(memo[id(memo)][0], x)
+
+    def test_deepcopy_dont_memo_immutable(self):
+        memo = {}
+        x = [1, 2, 3, 4]
+        y = copy.deepcopy(x, memo)
+        self.assertEqual(y, x)
+        # There's the entry for the new list, and the keep alive.
+        self.assertEqual(len(memo), 2)
+
+        memo = {}
+        x = [(1, 2)]
+        y = copy.deepcopy(x, memo)
+        self.assertEqual(y, x)
+        # Tuples with immutable contents are immutable for deepcopy.
+        self.assertEqual(len(memo), 2)
 
     def test_deepcopy_inst_vanilla(self):
         class C:
@@ -334,7 +374,7 @@ class TestCopy(unittest.TestCase):
         x = C([42])
         y = copy.deepcopy(x)
         self.assertEqual(y, x)
-        self.assertTrue(y.foo is not x.foo)
+        self.assertIsNot(y.foo, x.foo)
 
     def test_deepcopy_inst_deepcopy(self):
         class C:
@@ -347,8 +387,8 @@ class TestCopy(unittest.TestCase):
         x = C([42])
         y = copy.deepcopy(x)
         self.assertEqual(y, x)
-        self.assertTrue(y is not x)
-        self.assertTrue(y.foo is not x.foo)
+        self.assertIsNot(y, x)
+        self.assertIsNot(y.foo, x.foo)
 
     def test_deepcopy_inst_getinitargs(self):
         class C:
@@ -361,8 +401,8 @@ class TestCopy(unittest.TestCase):
         x = C([42])
         y = copy.deepcopy(x)
         self.assertEqual(y, x)
-        self.assertTrue(y is not x)
-        self.assertTrue(y.foo is not x.foo)
+        self.assertIsNot(y, x)
+        self.assertIsNot(y.foo, x.foo)
 
     def test_deepcopy_inst_getstate(self):
         class C:
@@ -375,8 +415,8 @@ class TestCopy(unittest.TestCase):
         x = C([42])
         y = copy.deepcopy(x)
         self.assertEqual(y, x)
-        self.assertTrue(y is not x)
-        self.assertTrue(y.foo is not x.foo)
+        self.assertIsNot(y, x)
+        self.assertIsNot(y.foo, x.foo)
 
     def test_deepcopy_inst_setstate(self):
         class C:
@@ -389,8 +429,8 @@ class TestCopy(unittest.TestCase):
         x = C([42])
         y = copy.deepcopy(x)
         self.assertEqual(y, x)
-        self.assertTrue(y is not x)
-        self.assertTrue(y.foo is not x.foo)
+        self.assertIsNot(y, x)
+        self.assertIsNot(y.foo, x.foo)
 
     def test_deepcopy_inst_getstate_setstate(self):
         class C:
@@ -405,8 +445,8 @@ class TestCopy(unittest.TestCase):
         x = C([42])
         y = copy.deepcopy(x)
         self.assertEqual(y, x)
-        self.assertTrue(y is not x)
-        self.assertTrue(y.foo is not x.foo)
+        self.assertIsNot(y, x)
+        self.assertIsNot(y.foo, x.foo)
 
     def test_deepcopy_reflexive_inst(self):
         class C:
@@ -414,8 +454,8 @@ class TestCopy(unittest.TestCase):
         x = C()
         x.foo = x
         y = copy.deepcopy(x)
-        self.assertTrue(y is not x)
-        self.assertTrue(y.foo is y)
+        self.assertIsNot(y, x)
+        self.assertIs(y.foo, y)
 
     # _reconstruct()
 
@@ -425,9 +465,9 @@ class TestCopy(unittest.TestCase):
                 return ""
         x = C()
         y = copy.copy(x)
-        self.assertTrue(y is x)
+        self.assertIs(y, x)
         y = copy.deepcopy(x)
-        self.assertTrue(y is x)
+        self.assertIs(y, x)
 
     def test_reconstruct_nostate(self):
         class C(object):
@@ -436,9 +476,9 @@ class TestCopy(unittest.TestCase):
         x = C()
         x.foo = 42
         y = copy.copy(x)
-        self.assertTrue(y.__class__ is x.__class__)
+        self.assertIs(y.__class__, x.__class__)
         y = copy.deepcopy(x)
-        self.assertTrue(y.__class__ is x.__class__)
+        self.assertIs(y.__class__, x.__class__)
 
     def test_reconstruct_state(self):
         class C(object):
@@ -452,7 +492,7 @@ class TestCopy(unittest.TestCase):
         self.assertEqual(y, x)
         y = copy.deepcopy(x)
         self.assertEqual(y, x)
-        self.assertTrue(y.foo is not x.foo)
+        self.assertIsNot(y.foo, x.foo)
 
     def test_reconstruct_state_setstate(self):
         class C(object):
@@ -468,7 +508,7 @@ class TestCopy(unittest.TestCase):
         self.assertEqual(y, x)
         y = copy.deepcopy(x)
         self.assertEqual(y, x)
-        self.assertTrue(y.foo is not x.foo)
+        self.assertIsNot(y.foo, x.foo)
 
     def test_reconstruct_reflexive(self):
         class C(object):
@@ -476,8 +516,8 @@ class TestCopy(unittest.TestCase):
         x = C()
         x.foo = x
         y = copy.deepcopy(x)
-        self.assertTrue(y is not x)
-        self.assertTrue(y.foo is y)
+        self.assertIsNot(y, x)
+        self.assertIs(y.foo, y)
 
     # Additions for Python 2.3 and pickle protocol 2
 
@@ -491,12 +531,12 @@ class TestCopy(unittest.TestCase):
         x = C([[1, 2], 3])
         y = copy.copy(x)
         self.assertEqual(x, y)
-        self.assertTrue(x is not y)
-        self.assertTrue(x[0] is y[0])
+        self.assertIsNot(x, y)
+        self.assertIs(x[0], y[0])
         y = copy.deepcopy(x)
         self.assertEqual(x, y)
-        self.assertTrue(x is not y)
-        self.assertTrue(x[0] is not y[0])
+        self.assertIsNot(x, y)
+        self.assertIsNot(x[0], y[0])
 
     def test_reduce_5tuple(self):
         class C(dict):
@@ -508,12 +548,12 @@ class TestCopy(unittest.TestCase):
         x = C([("foo", [1, 2]), ("bar", 3)])
         y = copy.copy(x)
         self.assertEqual(x, y)
-        self.assertTrue(x is not y)
-        self.assertTrue(x["foo"] is y["foo"])
+        self.assertIsNot(x, y)
+        self.assertIs(x["foo"], y["foo"])
         y = copy.deepcopy(x)
         self.assertEqual(x, y)
-        self.assertTrue(x is not y)
-        self.assertTrue(x["foo"] is not y["foo"])
+        self.assertIsNot(x, y)
+        self.assertIsNot(x["foo"], y["foo"])
 
     def test_copy_slots(self):
         class C(object):
@@ -521,7 +561,7 @@ class TestCopy(unittest.TestCase):
         x = C()
         x.foo = [42]
         y = copy.copy(x)
-        self.assertTrue(x.foo is y.foo)
+        self.assertIs(x.foo, y.foo)
 
     def test_deepcopy_slots(self):
         class C(object):
@@ -530,7 +570,7 @@ class TestCopy(unittest.TestCase):
         x.foo = [42]
         y = copy.deepcopy(x)
         self.assertEqual(x.foo, y.foo)
-        self.assertTrue(x.foo is not y.foo)
+        self.assertIsNot(x.foo, y.foo)
 
     def test_deepcopy_dict_subclass(self):
         class C(dict):
@@ -547,7 +587,7 @@ class TestCopy(unittest.TestCase):
         y = copy.deepcopy(x)
         self.assertEqual(x, y)
         self.assertEqual(x._keys, y._keys)
-        self.assertTrue(x is not y)
+        self.assertIsNot(x, y)
         x['bar'] = 1
         self.assertNotEqual(x, y)
         self.assertNotEqual(x._keys, y._keys)
@@ -560,8 +600,8 @@ class TestCopy(unittest.TestCase):
         y = copy.copy(x)
         self.assertEqual(list(x), list(y))
         self.assertEqual(x.foo, y.foo)
-        self.assertTrue(x[0] is y[0])
-        self.assertTrue(x.foo is y.foo)
+        self.assertIs(x[0], y[0])
+        self.assertIs(x.foo, y.foo)
 
     def test_deepcopy_list_subclass(self):
         class C(list):
@@ -571,8 +611,8 @@ class TestCopy(unittest.TestCase):
         y = copy.deepcopy(x)
         self.assertEqual(list(x), list(y))
         self.assertEqual(x.foo, y.foo)
-        self.assertTrue(x[0] is not y[0])
-        self.assertTrue(x.foo is not y.foo)
+        self.assertIsNot(x[0], y[0])
+        self.assertIsNot(x.foo, y.foo)
 
     def test_copy_tuple_subclass(self):
         class C(tuple):
@@ -589,8 +629,8 @@ class TestCopy(unittest.TestCase):
         self.assertEqual(tuple(x), ([1, 2], 3))
         y = copy.deepcopy(x)
         self.assertEqual(tuple(y), ([1, 2], 3))
-        self.assertTrue(x is not y)
-        self.assertTrue(x[0] is not y[0])
+        self.assertIsNot(x, y)
+        self.assertIsNot(x[0], y[0])
 
     def test_getstate_exc(self):
         class EvilState(object):
@@ -618,10 +658,10 @@ class TestCopy(unittest.TestCase):
         obj = C()
         x = weakref.ref(obj)
         y = _copy(x)
-        self.assertTrue(y is x)
+        self.assertIs(y, x)
         del obj
         y = _copy(x)
-        self.assertTrue(y is x)
+        self.assertIs(y, x)
 
     def test_copy_weakref(self):
         self._check_weakref(copy.copy)
@@ -637,7 +677,7 @@ class TestCopy(unittest.TestCase):
         u[a] = b
         u[c] = d
         v = copy.copy(u)
-        self.assertFalse(v is u)
+        self.assertIsNot(v, u)
         self.assertEqual(v, u)
         self.assertEqual(v[a], b)
         self.assertEqual(v[c], d)
@@ -647,7 +687,7 @@ class TestCopy(unittest.TestCase):
         x, y = C(), C()
         # The underlying containers are decoupled
         v[x] = y
-        self.assertFalse(x in u)
+        self.assertNotIn(x, u)
 
     def test_copy_weakkeydict(self):
         self._check_copy_weakdict(weakref.WeakKeyDictionary)
@@ -667,8 +707,8 @@ class TestCopy(unittest.TestCase):
         v = copy.deepcopy(u)
         self.assertNotEqual(v, u)
         self.assertEqual(len(v), 2)
-        self.assertFalse(v[a] is b)
-        self.assertFalse(v[c] is d)
+        self.assertIsNot(v[a], b)
+        self.assertIsNot(v[c], d)
         self.assertEqual(v[a].i, b.i)
         self.assertEqual(v[c].i, d.i)
         del c
@@ -687,16 +727,27 @@ class TestCopy(unittest.TestCase):
         self.assertNotEqual(v, u)
         self.assertEqual(len(v), 2)
         (x, y), (z, t) = sorted(v.items(), key=lambda pair: pair[0].i)
-        self.assertFalse(x is a)
+        self.assertIsNot(x, a)
         self.assertEqual(x.i, a.i)
-        self.assertTrue(y is b)
-        self.assertFalse(z is c)
+        self.assertIs(y, b)
+        self.assertIsNot(z, c)
         self.assertEqual(z.i, c.i)
-        self.assertTrue(t is d)
+        self.assertIs(t, d)
         del x, y, z, t
         del d
         self.assertEqual(len(v), 1)
 
+    def test_deepcopy_bound_method(self):
+        class Foo(object):
+            def m(self):
+                pass
+        f = Foo()
+        f.b = f.m
+        g = copy.deepcopy(f)
+        self.assertEqual(g.m, g.b)
+        self.assertIs(g.b.__self__, g)
+        g.b()
+
 
 def global_foo(x, y): return x+y
 
