@@ -38,26 +38,18 @@ STRINGLIB(fastsearch_memchr_1char)(const STRINGLIB_CHAR* s, Py_ssize_t n,
                                    STRINGLIB_CHAR ch, unsigned char needle,
                                    Py_ssize_t maxcount, int mode)
 {
-    void *candidate;
-    const STRINGLIB_CHAR *found;
-
-#define DO_MEMCHR(memchr, s, needle, nchars) do { \
-    candidate = memchr((const void *) (s), (needle), (nchars) * sizeof(STRINGLIB_CHAR)); \
-    found = (const STRINGLIB_CHAR *) \
-        ((Py_ssize_t) candidate & (~ ((Py_ssize_t) sizeof(STRINGLIB_CHAR) - 1))); \
-    } while (0)
-
     if (mode == FAST_SEARCH) {
         const STRINGLIB_CHAR *ptr = s;
         const STRINGLIB_CHAR *e = s + n;
         while (ptr < e) {
-            DO_MEMCHR(memchr, ptr, needle, e - ptr);
-            if (found == NULL)
+            void *candidate = memchr((const void *) ptr, needle, (e - ptr) * sizeof(STRINGLIB_CHAR));
+            if (candidate == NULL)
                 return -1;
-            if (sizeof(STRINGLIB_CHAR) == 1 || *found == ch)
-                return (found - s);
+            ptr = (const STRINGLIB_CHAR *) _Py_ALIGN_DOWN(candidate, sizeof(STRINGLIB_CHAR));
+            if (sizeof(STRINGLIB_CHAR) == 1 || *ptr == ch)
+                return (ptr - s);
             /* False positive */
-            ptr = found + 1;
+            ptr++;
         }
         return -1;
     }
@@ -67,9 +59,11 @@ STRINGLIB(fastsearch_memchr_1char)(const STRINGLIB_CHAR* s, Py_ssize_t n,
        faster than our hand-written loop in FASTSEARCH below */
     else if (mode == FAST_RSEARCH) {
         while (n > 0) {
-            DO_MEMCHR(memrchr, s, needle, n);
-            if (found == NULL)
+            const STRINGLIB_CHAR *found;
+            void *candidate = memrchr((const void *) s, needle, n * sizeof(STRINGLIB_CHAR));
+            if (candidate == NULL)
                 return -1;
+            found = (const STRINGLIB_CHAR *) _Py_ALIGN_DOWN(candidate, sizeof(STRINGLIB_CHAR));
             n = found - s;
             if (sizeof(STRINGLIB_CHAR) == 1 || *found == ch)
                 return n;
@@ -148,6 +142,8 @@ FASTSEARCH(const STRINGLIB_CHAR* s, Py_ssize_t n,
     mask = 0;
 
     if (mode != FAST_RSEARCH) {
+        const STRINGLIB_CHAR *ss = s + m - 1;
+        const STRINGLIB_CHAR *pp = p + m - 1;
 
         /* create compressed boyer-moore delta 1 table */
 
@@ -162,7 +158,7 @@ FASTSEARCH(const STRINGLIB_CHAR* s, Py_ssize_t n,
 
         for (i = 0; i <= w; i++) {
             /* note: using mlast in the skip path slows things down on x86 */
-            if (s[i+m-1] == p[m-1]) {
+            if (ss[i] == pp[0]) {
                 /* candidate match */
                 for (j = 0; j < mlast; j++)
                     if (s[i+j] != p[j])
@@ -178,13 +174,13 @@ FASTSEARCH(const STRINGLIB_CHAR* s, Py_ssize_t n,
                     continue;
                 }
                 /* miss: check if next character is part of pattern */
-                if (!STRINGLIB_BLOOM(mask, s[i+m]))
+                if (!STRINGLIB_BLOOM(mask, ss[i+1]))
                     i = i + m;
                 else
                     i = i + skip;
             } else {
                 /* skip: check if next character is part of pattern */
-                if (!STRINGLIB_BLOOM(mask, s[i+m]))
+                if (!STRINGLIB_BLOOM(mask, ss[i+1]))
                     i = i + m;
             }
         }
