@@ -50,8 +50,8 @@ PyDoc_STRVAR(module_doc,
 "FlushKey() - Writes all the attributes of the specified key to the registry.\n"
 "LoadKey() - Creates a subkey under HKEY_USER or HKEY_LOCAL_MACHINE and stores\n"
 "            registration information from a specified file into that subkey.\n"
-"OpenKey() - Alias for <om win32api.RegOpenKeyEx>\n"
-"OpenKeyEx() - Opens the specified key.\n"
+"OpenKey() - Opens the specified key.\n"
+"OpenKeyEx() - Alias of OpenKey().\n"
 "QueryValue() - Retrieves the value associated with the unnamed value for a\n"
 "               specified key in the registry.\n"
 "QueryValueEx() - Retrieves the type and data for a specified value name\n"
@@ -72,7 +72,8 @@ PyDoc_STRVAR(module_doc,
 
 
 PyDoc_STRVAR(CloseKey_doc,
-"CloseKey(hkey) - Closes a previously opened registry key.\n"
+"CloseKey(hkey)\n"
+"Closes a previously opened registry key.\n"
 "\n"
 "The hkey argument specifies a previously opened key.\n"
 "\n"
@@ -80,98 +81,145 @@ PyDoc_STRVAR(CloseKey_doc,
 "closed when the hkey object is destroyed by Python.");
 
 PyDoc_STRVAR(ConnectRegistry_doc,
-"key = ConnectRegistry(computer_name, key) - "
+"ConnectRegistry(computer_name, key) -> key\n"
 "Establishes a connection to a predefined registry handle on another computer.\n"
 "\n"
 "computer_name is the name of the remote computer, of the form \\\\computername.\n"
-" If None, the local computer is used.\n"
+"              If None, the local computer is used.\n"
 "key is the predefined handle to connect to.\n"
 "\n"
 "The return value is the handle of the opened key.\n"
-"If the function fails, an EnvironmentError exception is raised.");
+"If the function fails, an OSError exception is raised.");
 
 PyDoc_STRVAR(CreateKey_doc,
-"key = CreateKey(key, sub_key) - Creates or opens the specified key.\n"
+"CreateKey(key, sub_key) -> key\n"
+"Creates or opens the specified key.\n"
+"\n"
+"key is an already open key, or one of the predefined HKEY_* constants.\n"
+"sub_key is a string that names the key this method opens or creates.\n"
+"\n"
+"If key is one of the predefined keys, sub_key may be None. In that case,\n"
+"the handle returned is the same key handle passed in to the function.\n"
+"\n"
+"If the key already exists, this function opens the existing key.\n"
+"\n"
+"The return value is the handle of the opened key.\n"
+"If the function fails, an OSError exception is raised.");
+
+PyDoc_STRVAR(CreateKeyEx_doc,
+"CreateKeyEx(key, sub_key, reserved=0, access=KEY_WRITE) -> key\n"
+"Creates or opens the specified key.\n"
 "\n"
 "key is an already open key, or one of the predefined HKEY_* constants\n"
 "sub_key is a string that names the key this method opens or creates.\n"
-" If key is one of the predefined keys, sub_key may be None. In that case,\n"
-" the handle returned is the same key handle passed in to the function.\n"
+"reserved is a reserved integer, and must be zero.  Default is zero.\n"
+"access is an integer that specifies an access mask that describes the \n"
+"       desired security access for the key. Default is KEY_WRITE.\n"
+"\n"
+"If key is one of the predefined keys, sub_key may be None. In that case,\n"
+"the handle returned is the same key handle passed in to the function.\n"
 "\n"
 "If the key already exists, this function opens the existing key\n"
 "\n"
 "The return value is the handle of the opened key.\n"
-"If the function fails, an exception is raised.");
+"If the function fails, an OSError exception is raised.");
 
 PyDoc_STRVAR(DeleteKey_doc,
-"DeleteKey(key, sub_key) - Deletes the specified key.\n"
+"DeleteKey(key, sub_key)\n"
+"Deletes the specified key.\n"
 "\n"
 "key is an already open key, or any one of the predefined HKEY_* constants.\n"
-"sub_key is a string that must be a subkey of the key identified by the key parameter.\n"
-" This value must not be None, and the key may not have subkeys.\n"
+"sub_key is a string that must be a subkey of the key identified by the key\n"
+"        parameter. This value must not be None, and the key may not have\n"
+"        subkeys.\n"
 "\n"
 "This method can not delete keys with subkeys.\n"
 "\n"
-"If the method succeeds, the entire key, including all of its values,\n"
-"is removed.  If the method fails, an EnvironmentError exception is raised.");
+"If the function succeeds, the entire key, including all of its values,\n"
+"is removed.  If the function fails, an OSError exception is raised.");
+
+PyDoc_STRVAR(DeleteKeyEx_doc,
+"DeleteKeyEx(key, sub_key, access=KEY_WOW64_64KEY, reserved=0)\n"
+"Deletes the specified key (64-bit OS only).\n"
+"\n"
+"key is an already open key, or any one of the predefined HKEY_* constants.\n"
+"sub_key is a string that must be a subkey of the key identified by the key\n"
+"        parameter. This value must not be None, and the key may not have\n"
+"        subkeys.\n"
+"reserved is a reserved integer, and must be zero.  Default is zero.\n"
+"access is an integer that specifies an access mask that describes the \n"
+"       desired security access for the key. Default is KEY_WOW64_64KEY.\n"
+"\n"
+"This method can not delete keys with subkeys.\n"
+"\n"
+"If the function succeeds, the entire key, including all of its values,\n"
+"is removed.  If the function fails, an OSError exception is raised.\n"
+"On unsupported Windows versions, NotImplementedError is raised.");
 
 PyDoc_STRVAR(DeleteValue_doc,
-"DeleteValue(key, value) - Removes a named value from a registry key.\n"
+"DeleteValue(key, value)\n"
+"Removes a named value from a registry key.\n"
 "\n"
 "key is an already open key, or any one of the predefined HKEY_* constants.\n"
 "value is a string that identifies the value to remove.");
 
 PyDoc_STRVAR(EnumKey_doc,
-"string = EnumKey(key, index) - Enumerates subkeys of an open registry key.\n"
+"EnumKey(key, index) -> string\n"
+"Enumerates subkeys of an open registry key.\n"
 "\n"
 "key is an already open key, or any one of the predefined HKEY_* constants.\n"
 "index is an integer that identifies the index of the key to retrieve.\n"
 "\n"
 "The function retrieves the name of one subkey each time it is called.\n"
-"It is typically called repeatedly until an EnvironmentError exception is\n"
+"It is typically called repeatedly until an OSError exception is\n"
 "raised, indicating no more values are available.");
 
 PyDoc_STRVAR(EnumValue_doc,
-"tuple = EnumValue(key, index) - Enumerates values of an open registry key.\n"
+"EnumValue(key, index) -> tuple\n"
+"Enumerates values of an open registry key.\n"
 "key is an already open key, or any one of the predefined HKEY_* constants.\n"
 "index is an integer that identifies the index of the value to retrieve.\n"
 "\n"
 "The function retrieves the name of one subkey each time it is called.\n"
-"It is typically called repeatedly, until an EnvironmentError exception\n"
+"It is typically called repeatedly, until an OSError exception\n"
 "is raised, indicating no more values.\n"
 "\n"
 "The result is a tuple of 3 items:\n"
 "value_name is a string that identifies the value.\n"
 "value_data is an object that holds the value data, and whose type depends\n"
-" on the underlying registry type.\n"
+"           on the underlying registry type.\n"
 "data_type is an integer that identifies the type of the value data.");
 
 PyDoc_STRVAR(ExpandEnvironmentStrings_doc,
-"string = ExpandEnvironmentStrings(string) - Expand environment vars.\n");
+"ExpandEnvironmentStrings(string) -> string\n"
+"Expand environment vars.\n");
 
 PyDoc_STRVAR(FlushKey_doc,
-"FlushKey(key) - Writes all the attributes of a key to the registry.\n"
+"FlushKey(key)\n"
+"Writes all the attributes of a key to the registry.\n"
 "\n"
 "key is an already open key, or any one of the predefined HKEY_* constants.\n"
 "\n"
-"It is not necessary to call RegFlushKey to change a key.\n"
-"Registry changes are flushed to disk by the registry using its lazy flusher.\n"
-"Registry changes are also flushed to disk at system shutdown.\n"
-"Unlike CloseKey(), the FlushKey() method returns only when all the data has\n"
-"been written to the registry.\n"
-"An application should only call FlushKey() if it requires absolute certainty that registry changes are on disk.\n"
-"If you don't know whether a FlushKey() call is required, it probably isn't.");
+"It is not necessary to call FlushKey to change a key.  Registry changes are\n"
+"flushed to disk by the registry using its lazy flusher.  Registry changes are\n"
+"also flushed to disk at system shutdown.  Unlike CloseKey(), the FlushKey()\n"
+"method returns only when all the data has been written to the registry.\n"
+"\n"
+"An application should only call FlushKey() if it requires absolute certainty\n"
+"that registry changes are on disk.  If you don't know whether a FlushKey()\n"
+"call is required, it probably isn't.");
 
 PyDoc_STRVAR(LoadKey_doc,
-"LoadKey(key, sub_key, file_name) - Creates a subkey under the specified key\n"
-"and stores registration information from a specified file into that subkey.\n"
+"LoadKey(key, sub_key, file_name)\n"
+"Creates a subkey under the specified key and stores registration information\n"
+"from a specified file into that subkey.\n"
 "\n"
 "key is an already open key, or any one of the predefined HKEY_* constants.\n"
-"sub_key is a string that identifies the sub_key to load\n"
-"file_name is the name of the file to load registry data from.\n"
-" This file must have been created with the SaveKey() function.\n"
-" Under the file allocation table (FAT) file system, the filename may not\n"
-"have an extension.\n"
+"sub_key is a string that identifies the sub_key to load.\n"
+"file_name is the name of the file to load registry data from.  This file must\n"
+"          have been created with the SaveKey() function.  Under the file\n"
+"          allocation table (FAT) file system, the filename may not have an\n"
+"          extension.\n"
 "\n"
 "A call to LoadKey() fails if the calling process does not have the\n"
 "SE_RESTORE_PRIVILEGE privilege.\n"
@@ -182,69 +230,78 @@ PyDoc_STRVAR(LoadKey_doc,
 "The docs imply key must be in the HKEY_USER or HKEY_LOCAL_MACHINE tree");
 
 PyDoc_STRVAR(OpenKey_doc,
-"key = OpenKey(key, sub_key, res = 0, sam = KEY_READ) - Opens the specified key.\n"
+"OpenKey(key, sub_key, reserved=0, access=KEY_READ) -> key\n"
+"Opens the specified key.\n"
 "\n"
 "key is an already open key, or any one of the predefined HKEY_* constants.\n"
-"sub_key is a string that identifies the sub_key to open\n"
-"res is a reserved integer, and must be zero.  Default is zero.\n"
-"sam is an integer that specifies an access mask that describes the desired\n"
-" security access for the key.  Default is KEY_READ\n"
+"sub_key is a string that identifies the sub_key to open.\n"
+"reserved is a reserved integer, and must be zero.  Default is zero.\n"
+"access is an integer that specifies an access mask that describes the desired\n"
+"       security access for the key.  Default is KEY_READ\n"
 "\n"
 "The result is a new handle to the specified key\n"
-"If the function fails, an EnvironmentError exception is raised.");
+"If the function fails, an OSError exception is raised.");
 
 PyDoc_STRVAR(OpenKeyEx_doc, "See OpenKey()");
 
 PyDoc_STRVAR(QueryInfoKey_doc,
-"tuple = QueryInfoKey(key) - Returns information about a key.\n"
+"QueryInfoKey(key) -> tuple\n"
+"Returns information about a key.\n"
 "\n"
 "key is an already open key, or any one of the predefined HKEY_* constants.\n"
 "\n"
 "The result is a tuple of 3 items:"
 "An integer that identifies the number of sub keys this key has.\n"
 "An integer that identifies the number of values this key has.\n"
-"A long integer that identifies when the key was last modified (if available)\n"
+"An integer that identifies when the key was last modified (if available)\n"
 " as 100's of nanoseconds since Jan 1, 1600.");
 
 PyDoc_STRVAR(QueryValue_doc,
-"string = QueryValue(key, sub_key) - retrieves the unnamed value for a key.\n"
+"QueryValue(key, sub_key) -> string\n"
+"Retrieves the unnamed value for a key.\n"
 "\n"
 "key is an already open key, or any one of the predefined HKEY_* constants.\n"
 "sub_key is a string that holds the name of the subkey with which the value\n"
-" is associated.  If this parameter is None or empty, the function retrieves\n"
-" the value set by the SetValue() method for the key identified by key."
+"        is associated.  If this parameter is None or empty, the function\n"
+"        retrieves the value set by the SetValue() method for the key\n"
+"        identified by key."
 "\n"
 "Values in the registry have name, type, and data components. This method\n"
 "retrieves the data for a key's first value that has a NULL name.\n"
 "But the underlying API call doesn't return the type, Lame Lame Lame, DONT USE THIS!!!");
 
 PyDoc_STRVAR(QueryValueEx_doc,
-"value,type_id = QueryValueEx(key, value_name) - Retrieves the type and data for a specified value name associated with an open registry key.\n"
+"QueryValueEx(key, value_name) -> (value, type_id)\n"
+"Retrieves the type and data for a specified value name associated with an\n"
+"open registry key.\n"
 "\n"
 "key is an already open key, or any one of the predefined HKEY_* constants.\n"
 "value_name is a string indicating the value to query");
 
 PyDoc_STRVAR(SaveKey_doc,
-"SaveKey(key, file_name) - Saves the specified key, and all its subkeys to the specified file.\n"
+"SaveKey(key, file_name)\n"
+"Saves the specified key, and all its subkeys to the specified file.\n"
 "\n"
 "key is an already open key, or any one of the predefined HKEY_* constants.\n"
-"file_name is the name of the file to save registry data to.\n"
-" This file cannot already exist. If this filename includes an extension,\n"
-" it cannot be used on file allocation table (FAT) file systems by the\n"
-" LoadKey(), ReplaceKey() or RestoreKey() methods.\n"
+"file_name is the name of the file to save registry data to.  This file cannot\n"
+"          already exist. If this filename includes an extension, it cannot be\n"
+"          used on file allocation table (FAT) file systems by the LoadKey(),\n"
+"          ReplaceKey() or RestoreKey() methods.\n"
 "\n"
-"If key represents a key on a remote computer, the path described by\n"
-"file_name is relative to the remote computer.\n"
-"The caller of this method must possess the SeBackupPrivilege security privilege.\n"
-"This function passes NULL for security_attributes to the API.");
+"If key represents a key on a remote computer, the path described by file_name\n"
+"is relative to the remote computer.\n"
+"\n"
+"The caller of this method must possess the SeBackupPrivilege security\n"
+"privilege.  This function passes NULL for security_attributes to the API.");
 
 PyDoc_STRVAR(SetValue_doc,
-"SetValue(key, sub_key, type, value) - Associates a value with a specified key.\n"
+"SetValue(key, sub_key, type, value)\n"
+"Associates a value with a specified key.\n"
 "\n"
 "key is an already open key, or any one of the predefined HKEY_* constants.\n"
 "sub_key is a string that names the subkey with which the value is associated.\n"
-"type is an integer that specifies the type of the data.  Currently this\n"
-" must be REG_SZ, meaning only strings are supported.\n"
+"type is an integer that specifies the type of the data.  Currently this must\n"
+"     be REG_SZ, meaning only strings are supported.\n"
 "value is a string that specifies the new value.\n"
 "\n"
 "If the key specified by the sub_key parameter does not exist, the SetValue\n"
@@ -258,10 +315,12 @@ PyDoc_STRVAR(SetValue_doc,
 "KEY_SET_VALUE access.");
 
 PyDoc_STRVAR(SetValueEx_doc,
-"SetValueEx(key, value_name, reserved, type, value) - Stores data in the value field of an open registry key.\n"
+"SetValueEx(key, value_name, reserved, type, value)\n"
+"Stores data in the value field of an open registry key.\n"
 "\n"
 "key is an already open key, or any one of the predefined HKEY_* constants.\n"
-"value_name is a string containing the name of the value to set, or None\n"
+"value_name is a string containing the name of the value to set, or None.\n"
+"reserved can be anything - zero is always passed to the API.\n"
 "type is an integer that specifies the type of the data.  This should be one of:\n"
 "  REG_BINARY -- Binary data in any form.\n"
 "  REG_DWORD -- A 32-bit number.\n"
@@ -276,7 +335,6 @@ PyDoc_STRVAR(SetValueEx_doc,
 "  REG_NONE -- No defined value type.\n"
 "  REG_RESOURCE_LIST -- A device-driver resource list.\n"
 "  REG_SZ -- A null-terminated string.\n"
-"reserved can be anything - zero is always passed to the API.\n"
 "value is a string that specifies the new value.\n"
 "\n"
 "This method can also set additional value and type information for the\n"
@@ -293,6 +351,7 @@ PyDoc_STRVAR(DisableReflectionKey_doc,
 "Disables registry reflection for 32-bit processes running on a 64-bit\n"
 "Operating System.  Will generally raise NotImplemented if executed on\n"
 "a 32-bit Operating System.\n"
+"\n"
 "If the key is not on the reflection list, the function succeeds but has no effect.\n"
 "Disabling reflection for a key does not affect reflection of any subkeys.");
 
@@ -302,7 +361,8 @@ PyDoc_STRVAR(EnableReflectionKey_doc,
 "Restoring reflection for a key does not affect reflection of any subkeys.");
 
 PyDoc_STRVAR(QueryReflectionKey_doc,
-"bool = QueryReflectionKey(hkey) - Determines the reflection state for the specified key.\n"
+"QueryReflectionKey(hkey) -> bool\n"
+"Determines the reflection state for the specified key.\n"
 "Will generally raise NotImplemented if executed on a 32-bit Operating System.\n");
 
 /* PyHKEY docstrings */
@@ -330,12 +390,14 @@ PyDoc_STRVAR(PyHKEY_doc,
 
 
 PyDoc_STRVAR(PyHKEY_Close_doc,
-"key.Close() - Closes the underlying Windows handle.\n"
+"key.Close()\n"
+"Closes the underlying Windows handle.\n"
 "\n"
 "If the handle is already closed, no error is raised.");
 
 PyDoc_STRVAR(PyHKEY_Detach_doc,
-"int = key.Detach() - Detaches the Windows handle from the handle object.\n"
+"key.Detach() -> int\n"
+"Detaches the Windows handle from the handle object.\n"
 "\n"
 "The result is the value of the handle before it is detached.  If the\n"
 "handle is already detached, this will return zero.\n"
@@ -343,8 +405,7 @@ PyDoc_STRVAR(PyHKEY_Detach_doc,
 "After calling this function, the handle is effectively invalidated,\n"
 "but the handle is not closed.  You would call this function when you\n"
 "need the underlying win32 handle to exist beyond the lifetime of the\n"
-"handle object.\n"
-"On 64 bit windows, the result of this function is a long integer");
+"handle object.");
 
 
 /************************************************************************
@@ -422,7 +483,7 @@ PyHKEY_compareFunc(PyObject *ob1, PyObject *ob2)
          (pyhkey1 < pyhkey2 ? -1 : 1);
 }
 
-static long
+static Py_hash_t
 PyHKEY_hashFunc(PyObject *ob)
 {
     /* Just use the address.
@@ -723,32 +784,34 @@ Py2Reg(PyObject *value, DWORD typ, BYTE **retDataBuf, DWORD *retDataSize)
                 memcpy(*retDataBuf, &zero, sizeof(DWORD));
             }
             else {
-                DWORD d = PyLong_AsLong(value);
+                DWORD d = PyLong_AsUnsignedLong(value);
                 memcpy(*retDataBuf, &d, sizeof(DWORD));
             }
             break;
         case REG_SZ:
         case REG_EXPAND_SZ:
             {
-            if (value == Py_None)
-                *retDataSize = 1;
-            else {
-                if (!PyUnicode_Check(value))
-                    return FALSE;
-
-                *retDataSize = 2 + PyUnicode_GET_DATA_SIZE(value);
-            }
-            *retDataBuf = (BYTE *)PyMem_NEW(DWORD, *retDataSize);
-            if (*retDataBuf==NULL){
-                PyErr_NoMemory();
-                return FALSE;
-            }
-            if (value == Py_None)
-                wcscpy((wchar_t *)*retDataBuf, L"");
-            else
-                wcscpy((wchar_t *)*retDataBuf,
-                       PyUnicode_AS_UNICODE(value));
-            break;
+                if (value != Py_None) {
+                    Py_ssize_t len;
+                    if (!PyUnicode_Check(value))
+                        return FALSE;
+                    *retDataBuf = (BYTE*)PyUnicode_AsWideCharString(value, &len);
+                    if (*retDataBuf == NULL)
+                        return FALSE;
+                    *retDataSize = Py_SAFE_DOWNCAST(
+                        (len + 1) * sizeof(wchar_t),
+                        Py_ssize_t, DWORD);
+                }
+                else {
+                    *retDataBuf = (BYTE *)PyMem_NEW(wchar_t, 1);
+                    if (*retDataBuf == NULL) {
+                        PyErr_NoMemory();
+                        return FALSE;
+                    }
+                    ((wchar_t *)*retDataBuf)[0] = L'\0';
+                    *retDataSize = 1 * sizeof(wchar_t);
+                }
+                break;
             }
         case REG_MULTI_SZ:
             {
@@ -765,10 +828,17 @@ Py2Reg(PyObject *value, DWORD typ, BYTE **retDataBuf, DWORD *retDataSize)
                 for (j = 0; j < i; j++)
                 {
                     PyObject *t;
+                    wchar_t *wstr;
+                    Py_ssize_t len;
+
                     t = PyList_GET_ITEM(value, j);
                     if (!PyUnicode_Check(t))
                         return FALSE;
-                    size += 2 + PyUnicode_GET_DATA_SIZE(t);
+                    wstr = PyUnicode_AsUnicodeAndSize(t, &len);
+                    if (wstr == NULL)
+                        return FALSE;
+                    size += Py_SAFE_DOWNCAST((len + 1) * sizeof(wchar_t),
+                                             size_t, DWORD);
                 }
 
                 *retDataSize = size + 2;
@@ -783,10 +853,15 @@ Py2Reg(PyObject *value, DWORD typ, BYTE **retDataBuf, DWORD *retDataSize)
                 for (j = 0; j < i; j++)
                 {
                     PyObject *t;
+                    wchar_t *wstr;
+                    Py_ssize_t len;
+
                     t = PyList_GET_ITEM(value, j);
-                    wcscpy(P, PyUnicode_AS_UNICODE(t));
-                    P += 1 + wcslen(
-                        PyUnicode_AS_UNICODE(t));
+                    wstr = PyUnicode_AsUnicodeAndSize(t, &len);
+                    if (wstr == NULL)
+                        return FALSE;
+                    wcscpy(P, wstr);
+                    P += (len + 1);
                 }
                 /* And doubly-terminate the list... */
                 *P = '\0';
@@ -818,7 +893,7 @@ Py2Reg(PyObject *value, DWORD typ, BYTE **retDataBuf, DWORD *retDataSize)
                     PyErr_NoMemory();
                     return FALSE;
                 }
-                *retDataSize = view.len;
+                *retDataSize = Py_SAFE_DOWNCAST(view.len, Py_ssize_t, DWORD);
                 memcpy(*retDataBuf, view.buf, view.len);
                 PyBuffer_Release(&view);
             }
@@ -836,9 +911,9 @@ Reg2Py(BYTE *retDataBuf, DWORD retDataSize, DWORD typ)
     switch (typ) {
         case REG_DWORD:
             if (retDataSize == 0)
-                obData = PyLong_FromLong(0);
+                obData = PyLong_FromUnsignedLong(0);
             else
-                obData = PyLong_FromLong(*(int *)retDataBuf);
+                obData = PyLong_FromUnsignedLong(*(int *)retDataBuf);
             break;
         case REG_SZ:
         case REG_EXPAND_SZ:
@@ -850,7 +925,7 @@ Reg2Py(BYTE *retDataBuf, DWORD retDataSize, DWORD typ)
                     retDataSize -= 2;
                 if (retDataSize <= 0)
                     data = L"";
-                obData = PyUnicode_FromUnicode(data, retDataSize/2);
+                obData = PyUnicode_FromWideChar(data, retDataSize/2);
                 break;
             }
         case REG_MULTI_SZ:
@@ -862,7 +937,7 @@ Reg2Py(BYTE *retDataBuf, DWORD retDataSize, DWORD typ)
                 wchar_t *data = (wchar_t *)retDataBuf;
                 int len = retDataSize / 2;
                 int s = countStrings(data, len);
-                wchar_t **str = (wchar_t **)malloc(sizeof(wchar_t *)*s);
+                wchar_t **str = (wchar_t **)PyMem_Malloc(sizeof(wchar_t *)*s);
                 if (str == NULL)
                     return PyErr_NoMemory();
 
@@ -881,9 +956,9 @@ Reg2Py(BYTE *retDataBuf, DWORD retDataSize, DWORD typ)
                     }
                     PyList_SetItem(obData,
                                    index,
-                                   PyUnicode_FromUnicode(str[index], len));
+                                   PyUnicode_FromWideChar(str[index], len));
                 }
-                free(str);
+                PyMem_Free(str);
 
                 break;
             }
@@ -956,6 +1031,32 @@ PyCreateKey(PyObject *self, PyObject *args)
     return PyHKEY_FromHKEY(retKey);
 }
 
+static PyObject *
+PyCreateKeyEx(PyObject *self, PyObject *args, PyObject *kwargs)
+{
+    HKEY hKey;
+    PyObject *key;
+    wchar_t *sub_key;
+    HKEY retKey;
+    int reserved = 0;
+    REGSAM access = KEY_WRITE;
+    long rc;
+
+    char *kwlist[] = {"key", "sub_key", "reserved", "access", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "OZ|ii:CreateKeyEx", kwlist,
+                                     &key, &sub_key, &reserved, &access))
+        return NULL;
+    if (!PyHKEY_AsHKEY(key, &hKey, FALSE))
+        return NULL;
+
+    rc = RegCreateKeyExW(hKey, sub_key, reserved, NULL, (DWORD)NULL,
+                         access, NULL, &retKey, NULL);
+    if (rc != ERROR_SUCCESS)
+        return PyErr_SetFromWindowsErrWithFunction(rc, "CreateKeyEx");
+    return PyHKEY_FromHKEY(retKey);
+}
+
 static PyObject *
 PyDeleteKey(PyObject *self, PyObject *args)
 {
@@ -974,6 +1075,47 @@ PyDeleteKey(PyObject *self, PyObject *args)
     return Py_None;
 }
 
+static PyObject *
+PyDeleteKeyEx(PyObject *self, PyObject *args, PyObject *kwargs)
+{
+    HKEY hKey;
+    PyObject *key;
+    HMODULE hMod;
+    typedef LONG (WINAPI *RDKEFunc)(HKEY, const wchar_t*, REGSAM, int);
+    RDKEFunc pfn = NULL;
+    wchar_t *sub_key;
+    long rc;
+    int reserved = 0;
+    REGSAM access = KEY_WOW64_64KEY;
+
+    char *kwlist[] = {"key", "sub_key", "access", "reserved", NULL};
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "Ou|ii:DeleteKeyEx", kwlist,
+                                     &key, &sub_key, &access, &reserved))
+        return NULL;
+    if (!PyHKEY_AsHKEY(key, &hKey, FALSE))
+        return NULL;
+
+    /* Only available on 64bit platforms, so we must load it
+       dynamically. */
+    hMod = GetModuleHandleW(L"advapi32.dll");
+    if (hMod)
+        pfn = (RDKEFunc)GetProcAddress(hMod,
+                                                                   "RegDeleteKeyExW");
+    if (!pfn) {
+        PyErr_SetString(PyExc_NotImplementedError,
+                                        "not implemented on this platform");
+        return NULL;
+    }
+    Py_BEGIN_ALLOW_THREADS
+    rc = (*pfn)(hKey, sub_key, access, reserved);
+    Py_END_ALLOW_THREADS
+
+    if (rc != ERROR_SUCCESS)
+        return PyErr_SetFromWindowsErrWithFunction(rc, "RegDeleteKeyEx");
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyObject *
 PyDeleteValue(PyObject *self, PyObject *args)
 {
@@ -1011,7 +1153,7 @@ PyEnumKey(PyObject *self, PyObject *args)
      * nul.  RegEnumKeyEx requires a 257 character buffer to
      * retrieve such a key name. */
     wchar_t tmpbuf[257];
-    DWORD len = sizeof(tmpbuf); /* includes NULL terminator */
+    DWORD len = sizeof(tmpbuf)/sizeof(wchar_t); /* includes NULL terminator */
 
     if (!PyArg_ParseTuple(args, "Oi:EnumKey", &obKey, &index))
         return NULL;
@@ -1024,7 +1166,7 @@ PyEnumKey(PyObject *self, PyObject *args)
     if (rc != ERROR_SUCCESS)
         return PyErr_SetFromWindowsErrWithFunction(rc, "RegEnumKeyEx");
 
-    retStr = PyUnicode_FromUnicode(tmpbuf, len);
+    retStr = PyUnicode_FromWideChar(tmpbuf, len);
     return retStr;  /* can be NULL */
 }
 
@@ -1036,7 +1178,7 @@ PyEnumValue(PyObject *self, PyObject *args)
     int index;
     long rc;
     wchar_t *retValueBuf;
-    wchar_t *tmpBuf;
+    BYTE *tmpBuf;
     BYTE *retDataBuf;
     DWORD retValueSize, bufValueSize;
     DWORD retDataSize, bufDataSize;
@@ -1084,7 +1226,7 @@ PyEnumValue(PyObject *self, PyObject *args)
             break;
 
         bufDataSize *= 2;
-        tmpBuf = (wchar_t *)PyMem_Realloc(retDataBuf, bufDataSize);
+        tmpBuf = (BYTE *)PyMem_Realloc(retDataBuf, bufDataSize);
         if (tmpBuf == NULL) {
             PyErr_NoMemory();
             retVal = NULL;
@@ -1116,8 +1258,8 @@ PyEnumValue(PyObject *self, PyObject *args)
 static PyObject *
 PyExpandEnvironmentStrings(PyObject *self, PyObject *args)
 {
-    Py_UNICODE *retValue = NULL;
-    Py_UNICODE *src;
+    wchar_t *retValue = NULL;
+    wchar_t *src;
     DWORD retValueSize;
     DWORD rc;
     PyObject *o;
@@ -1130,7 +1272,7 @@ PyExpandEnvironmentStrings(PyObject *self, PyObject *args)
         return PyErr_SetFromWindowsErrWithFunction(retValueSize,
                                         "ExpandEnvironmentStrings");
     }
-    retValue = (Py_UNICODE *)PyMem_Malloc(retValueSize * sizeof(Py_UNICODE));
+    retValue = (wchar_t *)PyMem_Malloc(retValueSize * sizeof(wchar_t));
     if (retValue == NULL) {
         return PyErr_NoMemory();
     }
@@ -1141,7 +1283,7 @@ PyExpandEnvironmentStrings(PyObject *self, PyObject *args)
         return PyErr_SetFromWindowsErrWithFunction(retValueSize,
                                         "ExpandEnvironmentStrings");
     }
-    o = PyUnicode_FromUnicode(retValue, wcslen(retValue));
+    o = PyUnicode_FromWideChar(retValue, wcslen(retValue));
     PyMem_Free(retValue);
     return o;
 }
@@ -1187,24 +1329,26 @@ PyLoadKey(PyObject *self, PyObject *args)
 }
 
 static PyObject *
-PyOpenKey(PyObject *self, PyObject *args)
+PyOpenKey(PyObject *self, PyObject *args, PyObject *kwargs)
 {
     HKEY hKey;
-    PyObject *obKey;
-
-    wchar_t *subKey;
-    int res = 0;
+    PyObject *key;
+    wchar_t *sub_key;
+    int reserved = 0;
     HKEY retKey;
     long rc;
-    REGSAM sam = KEY_READ;
-    if (!PyArg_ParseTuple(args, "OZ|ii:OpenKey", &obKey, &subKey,
-                          &res, &sam))
+    REGSAM access = KEY_READ;
+
+    char *kwlist[] = {"key", "sub_key", "reserved", "access", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "OZ|ii:OpenKey", kwlist,
+                                     &key, &sub_key, &reserved, &access))
         return NULL;
-    if (!PyHKEY_AsHKEY(obKey, &hKey, FALSE))
+    if (!PyHKEY_AsHKEY(key, &hKey, FALSE))
         return NULL;
 
     Py_BEGIN_ALLOW_THREADS
-    rc = RegOpenKeyExW(hKey, subKey, res, sam, &retKey);
+    rc = RegOpenKeyExW(hKey, sub_key, reserved, access, &retKey);
     Py_END_ALLOW_THREADS
     if (rc != ERROR_SUCCESS)
         return PyErr_SetFromWindowsErrWithFunction(rc, "RegOpenKeyEx");
@@ -1293,7 +1437,7 @@ PyQueryValue(PyObject *self, PyObject *args)
                                                    "RegQueryValue");
     }
 
-    retStr = PyUnicode_FromUnicode(retBuf, wcslen(retBuf));
+    retStr = PyUnicode_FromWideChar(retBuf, wcslen(retBuf));
     PyMem_Free(retBuf);
     return retStr;
 }
@@ -1424,7 +1568,7 @@ PySetValueEx(PyObject *self, PyObject *args)
 {
     HKEY hKey;
     PyObject *obKey;
-    Py_UNICODE *valueName;
+    wchar_t *valueName;
     PyObject *obRes;
     PyObject *value;
     BYTE *data;
@@ -1475,9 +1619,9 @@ PyDisableReflectionKey(PyObject *self, PyObject *args)
     if (!PyHKEY_AsHKEY(obKey, &hKey, FALSE))
         return NULL;
 
-    // Only available on 64bit platforms, so we must load it
-    // dynamically.
-    hMod = GetModuleHandle("advapi32.dll");
+    /* Only available on 64bit platforms, so we must load it
+       dynamically.*/
+    hMod = GetModuleHandleW(L"advapi32.dll");
     if (hMod)
         pfn = (RDRKFunc)GetProcAddress(hMod,
                                        "RegDisableReflectionKey");
@@ -1511,9 +1655,9 @@ PyEnableReflectionKey(PyObject *self, PyObject *args)
     if (!PyHKEY_AsHKEY(obKey, &hKey, FALSE))
         return NULL;
 
-    // Only available on 64bit platforms, so we must load it
-    // dynamically.
-    hMod = GetModuleHandle("advapi32.dll");
+    /* Only available on 64bit platforms, so we must load it
+       dynamically.*/
+    hMod = GetModuleHandleW(L"advapi32.dll");
     if (hMod)
         pfn = (RERKFunc)GetProcAddress(hMod,
                                        "RegEnableReflectionKey");
@@ -1548,9 +1692,9 @@ PyQueryReflectionKey(PyObject *self, PyObject *args)
     if (!PyHKEY_AsHKEY(obKey, &hKey, FALSE))
         return NULL;
 
-    // Only available on 64bit platforms, so we must load it
-    // dynamically.
-    hMod = GetModuleHandle("advapi32.dll");
+    /* Only available on 64bit platforms, so we must load it
+       dynamically.*/
+    hMod = GetModuleHandleW(L"advapi32.dll");
     if (hMod)
         pfn = (RQRKFunc)GetProcAddress(hMod,
                                        "RegQueryReflectionKey");
@@ -1565,14 +1709,18 @@ PyQueryReflectionKey(PyObject *self, PyObject *args)
     if (rc != ERROR_SUCCESS)
         return PyErr_SetFromWindowsErrWithFunction(rc,
                                                    "RegQueryReflectionKey");
-    return PyBool_FromLong(rc);
+    return PyBool_FromLong(result);
 }
 
 static struct PyMethodDef winreg_methods[] = {
     {"CloseKey",         PyCloseKey,        METH_VARARGS, CloseKey_doc},
     {"ConnectRegistry",  PyConnectRegistry, METH_VARARGS, ConnectRegistry_doc},
     {"CreateKey",        PyCreateKey,       METH_VARARGS, CreateKey_doc},
+    {"CreateKeyEx",      (PyCFunction)PyCreateKeyEx,
+                         METH_VARARGS | METH_KEYWORDS, CreateKeyEx_doc},
     {"DeleteKey",        PyDeleteKey,       METH_VARARGS, DeleteKey_doc},
+    {"DeleteKeyEx",      (PyCFunction)PyDeleteKeyEx,
+                         METH_VARARGS | METH_KEYWORDS, DeleteKeyEx_doc},
     {"DeleteValue",      PyDeleteValue,     METH_VARARGS, DeleteValue_doc},
     {"DisableReflectionKey", PyDisableReflectionKey, METH_VARARGS, DisableReflectionKey_doc},
     {"EnableReflectionKey",  PyEnableReflectionKey,  METH_VARARGS, EnableReflectionKey_doc},
@@ -1582,8 +1730,10 @@ static struct PyMethodDef winreg_methods[] = {
         ExpandEnvironmentStrings_doc },
     {"FlushKey",         PyFlushKey,        METH_VARARGS, FlushKey_doc},
     {"LoadKey",          PyLoadKey,         METH_VARARGS, LoadKey_doc},
-    {"OpenKey",          PyOpenKey,         METH_VARARGS, OpenKey_doc},
-    {"OpenKeyEx",        PyOpenKey,         METH_VARARGS, OpenKeyEx_doc},
+    {"OpenKey",          (PyCFunction)PyOpenKey, METH_VARARGS | METH_KEYWORDS,
+                                                 OpenKey_doc},
+    {"OpenKeyEx",        (PyCFunction)PyOpenKey, METH_VARARGS | METH_KEYWORDS,
+                                                 OpenKeyEx_doc},
     {"QueryValue",       PyQueryValue,      METH_VARARGS, QueryValue_doc},
     {"QueryValueEx",     PyQueryValueEx,    METH_VARARGS, QueryValueEx_doc},
     {"QueryInfoKey",     PyQueryInfoKey,    METH_VARARGS, QueryInfoKey_doc},
@@ -1643,9 +1793,9 @@ PyMODINIT_FUNC PyInit_winreg(void)
     if (PyDict_SetItemString(d, "HKEYType",
                              (PyObject *)&PyHKEY_Type) != 0)
         return NULL;
-    Py_INCREF(PyExc_WindowsError);
+    Py_INCREF(PyExc_OSError);
     if (PyDict_SetItemString(d, "error",
-                             PyExc_WindowsError) != 0)
+                             PyExc_OSError) != 0)
         return NULL;
 
     /* Add the relevant constants */
