@@ -1,4 +1,4 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 
 "Replace LF with CRLF in argument files.  Print names of changed files."
 
@@ -9,16 +9,16 @@ def main():
         if os.path.isdir(filename):
             print(filename, "Directory!")
             continue
-        data = open(filename, "rb").read()
-        if '\0' in data:
+        with open(filename, "rb") as f:
+            data = f.read()
+        if b'\0' in data:
             print(filename, "Binary!")
             continue
-        newdata = re.sub("\r?\n", "\r\n", data)
+        newdata = re.sub(b"\r?\n", b"\r\n", data)
         if newdata != data:
             print(filename)
-            f = open(filename, "wb")
-            f.write(newdata)
-            f.close()
+            with open(filename, "wb") as f:
+                f.write(newdata)
 
 if __name__ == '__main__':
     main()
