@@ -124,13 +124,13 @@ def docs_modified(file_paths):
 @status("Misc/ACKS updated", modal=True)
 def credit_given(file_paths):
     """Check if Misc/ACKS has been changed."""
-    return 'Misc/ACKS' in file_paths
+    return os.path.join('Misc', 'ACKS') in file_paths
 
 
 @status("Misc/NEWS updated", modal=True)
 def reported_news(file_paths):
     """Check if Misc/NEWS has been changed."""
-    return 'Misc/NEWS' in file_paths
+    return os.path.join('Misc', 'NEWS') in file_paths
 
 @status("configure regenerated", modal=True, info=str)
 def regenerated_configure(file_paths):
@@ -153,7 +153,8 @@ def main():
     python_files = [fn for fn in file_paths if fn.endswith('.py')]
     c_files = [fn for fn in file_paths if fn.endswith(('.c', '.h'))]
     doc_files = [fn for fn in file_paths if fn.startswith('Doc')]
-    special_files = {'Misc/ACKS', 'Misc/NEWS'} & set(file_paths)
+    misc_files = {os.path.join('Misc', 'ACKS'), os.path.join('Misc', 'NEWS')}\
+            & set(file_paths)
     # PEP 8 whitespace rules enforcement.
     normalize_whitespace(python_files)
     # C rules enforcement.
@@ -163,9 +164,9 @@ def main():
     # Docs updated.
     docs_modified(doc_files)
     # Misc/ACKS changed.
-    credit_given(special_files)
+    credit_given(misc_files)
     # Misc/NEWS changed.
-    reported_news(special_files)
+    reported_news(misc_files)
     # Regenerated configure, if necessary.
     regenerated_configure(file_paths)
     # Regenerated pyconfig.h.in, if necessary.
@@ -173,8 +174,9 @@ def main():
 
     # Test suite run and passed.
     if python_files or c_files:
+        end = " and check for refleaks?" if c_files else "?"
         print()
-        print("Did you run the test suite?")
+        print("Did you run the test suite" + end)
 
 
 if __name__ == '__main__':
